@@ -2,7 +2,6 @@
 from cms.admin.dialog.forms import PermissionForm
 from cms.models import Page
 from cms.utils import get_cms_setting
-from django.conf import settings
 from django.contrib.admin.views.decorators import staff_member_required
 from django.http import Http404, HttpResponse
 from django.shortcuts import render_to_response, get_object_or_404
@@ -21,7 +20,7 @@ def get_copy_dialog(request, page_id):
     
     context = {
         'dialog_id': 'dialog-copy',
-        'form': PermissionForm(), # class needs to be instantiated
+        'form': PermissionForm(),  # class needs to be instantiated
         'callback': request.REQUEST['callback'],
     }
-    return render_to_response("admin/cms/page/dialog/copy.html", context)
+    return render_to_response("admin/cms/page/tree/copy_premissions.html", context)
\ No newline at end of file
