@@ -1,20 +1,19 @@
 # -*- coding: utf-8 -*-
-from cms import constants
 from cms.apphook_pool import apphook_pool
-from cms.forms.widgets import UserSelectAdminWidget
-from cms.models import (Page, PagePermission, PageUser, ACCESS_PAGE, 
-    PageUserGroup, titlemodels)
+from cms.compat import get_user_model
+from cms.compat_forms import UserCreationForm
+from cms.constants import PAGE_TYPES_ID
+from cms.forms.widgets import UserSelectAdminWidget, AppHookSelect
+from cms.models import Page, PagePermission, PageUser, ACCESS_PAGE, PageUserGroup, Title, EmptyTitle
 from cms.utils.conf import get_cms_setting
 from cms.utils.i18n import get_language_tuple, get_language_list
 from cms.utils.mail import mail_page_user_change
 from cms.utils.page import is_valid_page_slug
-from cms.utils.page_resolver import get_page_from_path, is_valid_url
-from cms.utils.permissions import (get_current_user, get_subordinate_users, 
-    get_subordinate_groups, get_user_permission_level)
-from cms.utils.urlutils import any_path_re
+from cms.utils.page_resolver import is_valid_url
+from cms.utils.permissions import get_current_user, get_subordinate_users, get_subordinate_groups, \
+    get_user_permission_level
 from django import forms
-from django.contrib.auth.forms import UserCreationForm
-from django.contrib.auth.models import Permission, User
+from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.sites.models import Site
 from django.core.exceptions import ValidationError
@@ -22,19 +21,21 @@ from django.db.models.fields import BooleanField
 from django.forms.util import ErrorList
 from django.forms.widgets import HiddenInput
 from django.template.defaultfilters import slugify
+from cms.utils.compat.dj import force_unicode
 from django.utils.translation import ugettext_lazy as _, get_language
 from menus.menu_pool import menu_pool
 
 
-
-
 def get_permission_acessor(obj):
+    User = get_user_model()
+    
     if isinstance(obj, (PageUser, User,)):
         rel_name = 'user_permissions'
     else:
         rel_name = 'permissions'
     return getattr(obj, rel_name)
 
+
 def save_permissions(data, obj):
     models = (
         (Page, 'page'),
@@ -57,22 +58,35 @@ def save_permissions(data, obj):
             else:
                 permission_acessor.remove(permission)
 
-class PageAddForm(forms.ModelForm):
-    title = forms.CharField(label=_("Title"), widget=forms.TextInput(),
-        help_text=_('The default title'))
-    slug = forms.CharField(label=_("Slug"), widget=forms.TextInput(),
-        help_text=_('The part of the title that is used in the URL'))
+
+class PageForm(forms.ModelForm):
     language = forms.ChoiceField(label=_("Language"), choices=get_language_tuple(),
-        help_text=_('The current language of the content fields.'))
-    
+                                 help_text=_('The current language of the content fields.'))
+    page_type = forms.ChoiceField(label=_("Page type"), required=False)
+    title = forms.CharField(label=_("Title"), widget=forms.TextInput(),
+                            help_text=_('The default title'))
+    slug = forms.CharField(label=_("Slug"), widget=forms.TextInput(),
+                           help_text=_('The part of the title that is used in the URL'))
+    menu_title = forms.CharField(label=_("Menu Title"), widget=forms.TextInput(),
+                                 help_text=_('Overwrite what is displayed in the menu'), required=False)
+    page_title = forms.CharField(label=_("Page Title"), widget=forms.TextInput(),
+                                 help_text=_('Overwrites what is displayed at the top of your browser or in bookmarks'),
+                                 required=False)
+    meta_description = forms.CharField(label=_('Description meta tag'), required=False,
+                                       widget=forms.Textarea(attrs={'maxlength': '155', 'rows': '4'}),
+                                       help_text=_('A description of the page used by search engines.'),
+                                       max_length=155)
+
     class Meta:
         model = Page
-        exclude = ["created_by", "changed_by", "placeholders"]
-    
+        fields = ["parent", "site", 'template']
+
     def __init__(self, *args, **kwargs):
-        super(PageAddForm, self).__init__(*args, **kwargs)
-        self.fields['parent'].widget = HiddenInput() 
+        super(PageForm, self).__init__(*args, **kwargs)
+        self.fields['parent'].widget = HiddenInput()
         self.fields['site'].widget = HiddenInput()
+        self.fields['template'].widget = HiddenInput()
+        self.fields['language'].widget = HiddenInput()
         if not self.fields['site'].initial:
             self.fields['site'].initial = Site.objects.get_current().pk
         site_id = self.fields['site'].initial
@@ -80,12 +94,20 @@ class PageAddForm(forms.ModelForm):
         self.fields['language'].choices = languages
         if not self.fields['language'].initial:
             self.fields['language'].initial = get_language()
-        if (self.fields['parent'].initial and
-            get_cms_setting('TEMPLATE_INHERITANCE') in
-            [name for name, value in get_cms_setting('TEMPLATES')]):
-            # non-root pages default to inheriting their template
-            self.fields['template'].initial = constants.TEMPLATE_INHERITANCE_MAGIC
-        
+        if 'page_type' in self.fields:
+            try:
+                type_root = Page.objects.get(publisher_is_draft=True, reverse_id=PAGE_TYPES_ID)
+            except Page.DoesNotExist:
+                type_root = None
+            if type_root:
+                language = self.fields['language'].initial
+                type_ids = type_root.get_descendants().values_list('pk', flat=True)
+                titles = Title.objects.filter(page__in=type_ids, language=language)
+                choices = [('', '----')]
+                for title in titles:
+                    choices.append((title.page_id, title.title))
+                self.fields['page_type'].choices = choices
+
     def clean(self):
         cleaned_data = self.cleaned_data
         slug = cleaned_data.get('slug', '')
@@ -93,102 +115,216 @@ class PageAddForm(forms.ModelForm):
         page = self.instance
         lang = cleaned_data.get('language', None)
         # No language, can not go further, but validation failed already
-        if not lang: 
+        if not lang:
             return cleaned_data
-        
+
         if 'parent' not in cleaned_data:
             cleaned_data['parent'] = None
         parent = cleaned_data.get('parent', None)
-        
+
         try:
             site = self.cleaned_data.get('site', Site.objects.get_current())
         except Site.DoesNotExist:
-            site = None
             raise ValidationError("No site found for current settings.")
 
         if parent and parent.site != site:
             raise ValidationError("Site doesn't match the parent's page site")
-        
+
         if site and not is_valid_page_slug(page, parent, lang, slug, site):
             self._errors['slug'] = ErrorList([_('Another page with this slug already exists')])
             del cleaned_data['slug']
-        if self.cleaned_data.get('published') and page.title_set.count():
+        if self.instance and page.title_set.count():
             #Check for titles attached to the page makes sense only because
             #AdminFormsTests.test_clean_overwrite_url validates the form with when no page instance available
             #Looks like just a theoretical corner case
-            try:
-                title = page.get_title_obj(lang, fallback=False)
-            except titlemodels.Title.DoesNotExist:
-                title = None
-            if title and not isinstance(title, titlemodels.EmptyTitle) and slug:
+            title = page.get_title_obj(lang, fallback=False)
+            if title and not isinstance(title, EmptyTitle) and slug:
                 oldslug = title.slug
                 title.slug = slug
                 title.save()
                 try:
-                    is_valid_url(title.path,page)
-                except ValidationError,e:
+                    is_valid_url(title.path, page)
+                except ValidationError as exc:
                     title.slug = oldslug
                     title.save()
                     if 'slug' in cleaned_data:
                         del cleaned_data['slug']
-                    self._errors['slug'] = ErrorList(e.messages)
+                    if hasattr(exc, 'messages'):
+                        errors = exc.messages
+                    else:
+                        errors = [force_unicode(exc.message)]
+                    self._errors['slug'] = ErrorList(errors)
         return cleaned_data
-    
+
     def clean_slug(self):
         slug = slugify(self.cleaned_data['slug'])
         if not slug:
-            raise ValidationError("Slug must not be empty.")
+            raise ValidationError(_("Slug must not be empty."))
         return slug
-    
+
     def clean_language(self):
         language = self.cleaned_data['language']
         if not language in get_language_list():
             raise ValidationError("Given language does not match language settings.")
         return language
-        
-    
-class PageForm(PageAddForm):
-    menu_title = forms.CharField(label=_("Menu Title"), widget=forms.TextInput(),
-        help_text=_('Overwrite what is displayed in the menu'), required=False)
-    page_title = forms.CharField(label=_("Page Title"), widget=forms.TextInput(),
-        help_text=_('Overwrites what is displayed at the top of your browser or in bookmarks'), required=False)
-    application_urls = forms.ChoiceField(label=_('Application'), 
-        choices=(), required=False,  
-        help_text=_('Hook application to this page.'))
-    overwrite_url = forms.CharField(label=_('Overwrite URL'), max_length=255, required=False,
-        help_text=_('Keep this field empty if standard path should be used.'))
 
-    redirect = forms.CharField(label=_('Redirect'), max_length=255, required=False,
-        help_text=_('Redirects to this URL.'))
-    meta_description = forms.CharField(label='Description meta tag', required=False, widget=forms.Textarea,
-        help_text=_('A description of the page sometimes used by search engines.'))
-    meta_keywords = forms.CharField(label='Keywords meta tag', max_length=255, required=False,
-        help_text=_('A list of comma seperated keywords sometimes used by search engines.'))
-    
+
+class PublicationDatesForm(forms.ModelForm):
+    language = forms.ChoiceField(label=_("Language"), choices=get_language_tuple(),
+                                 help_text=_('The current language of the content fields.'))
+
     def __init__(self, *args, **kwargs):
-        super(PageForm, self).__init__(*args, **kwargs)
+        # Dates are not language dependent, so let's just fake the language to
+        # make the ModelAdmin happy
+        super(PublicationDatesForm, self).__init__(*args, **kwargs)
+        self.fields['language'].widget = HiddenInput()
+        self.fields['site'].widget = HiddenInput()
+        site_id = self.fields['site'].initial
+
+        languages = get_language_tuple(site_id)
+        self.fields['language'].choices = languages
+        if not self.fields['language'].initial:
+            self.fields['language'].initial = get_language()
+
+    class Meta:
+        model = Page
+        fields = ['site', 'publication_date', 'publication_end_date']
+
+
+class AdvancedSettingsForm(forms.ModelForm):
+    from cms.forms.fields import PageSmartLinkField
+    application_urls = forms.ChoiceField(label=_('Application'),
+                                         choices=(), required=False,
+                                         help_text=_('Hook application to this page.'))
+    overwrite_url = forms.CharField(label=_('Overwrite URL'), max_length=255, required=False,
+                                    help_text=_('Keep this field empty if standard path should be used.'))
+
+    xframe_options = forms.ChoiceField(
+        choices=Page._meta.get_field('xframe_options').choices,
+        label=_('X Frame Options'),
+        help_text=_('Whether this page can be embedded in other pages or websites'),
+        initial=Page._meta.get_field('xframe_options').default,
+        required=False
+    )
+
+    redirect = PageSmartLinkField(label=_('Redirect'), required=False,
+                               help_text=_('Redirects to this URL.'), placeholder_text=_('Start typing...'))
+
+    language = forms.ChoiceField(label=_("Language"), choices=get_language_tuple(),
+                                 help_text=_('The current language of the content fields.'))
+
+    def __init__(self, *args, **kwargs):
+        super(AdvancedSettingsForm, self).__init__(*args, **kwargs)
+        self.fields['language'].widget = HiddenInput()
+        self.fields['site'].widget = HiddenInput()
+        site_id = self.fields['site'].initial
+
+        languages = get_language_tuple(site_id)
+        self.fields['language'].choices = languages
+        if not self.fields['language'].initial:
+            self.fields['language'].initial = get_language()
         if 'navigation_extenders' in self.fields:
-            self.fields['navigation_extenders'].widget = forms.Select({}, [('', "---------")] + menu_pool.get_menus_by_attribute("cms_enabled", True))
+            self.fields['navigation_extenders'].widget = forms.Select({},
+                [('', "---------")] + menu_pool.get_menus_by_attribute("cms_enabled", True))
         if 'application_urls' in self.fields:
+            # Prepare a dict mapping the apps by class name ('PollApp') to
+            # their app_name attribute ('polls'), if any.
+            app_namespaces = {}
+            for app_class in apphook_pool.apps.keys():
+                app = apphook_pool.apps[app_class]
+                if app.app_name:
+                    app_namespaces[app_class] = app.app_name
+
+            self.fields['application_urls'].widget = AppHookSelect(
+                attrs={'id':'application_urls'},
+                app_namespaces=app_namespaces,
+            )
             self.fields['application_urls'].choices = [('', "---------")] + apphook_pool.get_apphooks()
-            
+
+        if 'redirect' in self.fields:
+            self.fields['redirect'].widget.language = self.fields['language'].initial
+
     def clean(self):
-        cleaned_data = super(PageForm, self).clean()
+        cleaned_data = super(AdvancedSettingsForm, self).clean()
         if 'reverse_id' in self.fields:
             id = cleaned_data['reverse_id']
             site_id = cleaned_data['site']
             if id:
-                if Page.objects.filter(reverse_id=id, site=site_id, publisher_is_draft=True).exclude(pk=self.instance.pk).count():
-                    self._errors['reverse_id'] = self.error_class([_('A page with this reverse URL id exists already.')])
+                if Page.objects.filter(reverse_id=id, site=site_id, publisher_is_draft=True).exclude(
+                        pk=self.instance.pk).count():
+                    self._errors['reverse_id'] = self.error_class(
+                        [_('A page with this reverse URL id exists already.')])
+        apphook = cleaned_data.get('application_urls', None)
+        # The field 'application_namespace' is a misnomer. It should be
+        # 'instance_namespace'.
+        instance_namespace = cleaned_data.get('application_namespace', None)
+        if apphook:
+            apphook_pool.discover_apps()
+            # The attribute on the apps 'app_name' is a misnomer, it should be
+            # 'application_namespace'.
+            application_namespace = apphook_pool.apps[apphook].app_name
+            if application_namespace and not instance_namespace:
+                if Page.objects.filter(
+                    publisher_is_draft=True,
+                    application_urls=apphook,
+                    application_namespace=application_namespace
+                ).exclude(pk=self.instance.pk).count():
+                    # Looks like there's already one with the default instance
+                    # namespace defined.
+                    self._errors['application_urls'] = ErrorList([
+                        _('''You selected an apphook with an "app_name".
+                            You must enter a unique instance name.''')
+                    ])
+                else:
+                    # OK, there are zero instances of THIS app that use the
+                    # default instance namespace, so, since the user didn't
+                    # provide one, we'll use the default. NOTE: The following
+                    # line is really setting the "instance namespace" of the
+                    # new app to the app’s "application namespace", which is
+                    # the default instance namespace.
+                    self.cleaned_data['application_namespace'] = application_namespace
+
+        if instance_namespace and not apphook:
+            self.cleaned_data['application_namespace'] = None
+
         return cleaned_data
 
+    def clean_application_namespace(self):
+        namespace = self.cleaned_data['application_namespace']
+        if namespace and Page.objects.filter(publisher_is_draft=True, application_namespace=namespace).exclude(pk=self.instance.pk).count():
+            raise ValidationError(_('A instance name with this name already exists.'))
+        return namespace
+
+    def clean_xframe_options(self):
+        if 'xframe_options' not in self.fields:
+            return # nothing to do, field isn't present
+
+        xframe_options = self.cleaned_data['xframe_options']
+        if xframe_options == '':
+            return Page._meta.get_field('xframe_options').default
+
+        return xframe_options
+
     def clean_overwrite_url(self):
         if 'overwrite_url' in self.fields:
             url = self.cleaned_data['overwrite_url']
-            is_valid_url(url,self.instance)
-            # TODO: Check what happens if 'overwrite_url' is NOT in self.fields
+            is_valid_url(url, self.instance)
             return url
 
+    class Meta:
+        model = Page
+        fields = [
+            'site', 'template', 'reverse_id', 'overwrite_url', 'redirect', 'soft_root', 'navigation_extenders',
+            'application_urls', 'application_namespace', "xframe_options",
+        ]
+
+
+class PagePermissionForm(forms.ModelForm):
+    class Meta:
+        model = Page
+        fields = ['login_required', 'limit_visibility_in_menu']
+
+
 class PagePermissionInlineAdminForm(forms.ModelForm):
     """
     Page permission inline admin form used in inline admin. Required, because
@@ -197,7 +333,7 @@ class PagePermissionInlineAdminForm(forms.ModelForm):
     but aren't assigned to higher page level than current user.
     """
     page = forms.ModelChoiceField(Page, label=_('user'), widget=HiddenInput(), required=True)
-    
+
     def __init__(self, *args, **kwargs):
         super(PagePermissionInlineAdminForm, self).__init__(*args, **kwargs)
         user = get_current_user() # current user from threadlocals
@@ -244,7 +380,7 @@ class PagePermissionInlineAdminForm(forms.ModelForm):
             self.fields['user'].widget.user = user # assign current user
 
         self.fields['group'].queryset = get_subordinate_groups(user)
-    
+
     def clean(self):
         super(PagePermissionInlineAdminForm, self).clean()
         for field in self.Meta.model._meta.fields:
@@ -252,7 +388,7 @@ class PagePermissionInlineAdminForm(forms.ModelForm):
                 continue
             name = field.name
             self.cleaned_data[name] = self.cleaned_data.get(name, False)
-        
+
         can_add = self.cleaned_data['can_add']
         can_edit = self.cleaned_data['can_change']
         # check if access for childrens, or descendants is granted
@@ -261,19 +397,19 @@ class PagePermissionInlineAdminForm(forms.ModelForm):
             # page but after he does this, he will not have permissions to 
             # access this page anymore, so avoid this
             raise forms.ValidationError(_("Add page permission requires also "
-                "access to children, or descendants, otherwise added page "
-                "can't be changed by its creator."))
-        
+                                          "access to children, or descendants, otherwise added page "
+                                          "can't be changed by its creator."))
+
         if can_add and not can_edit:
             raise forms.ValidationError(_('Add page permission also requires edit page permission.'))
-        # TODO: finish this, but is it really required? might be nice to have 
-        
+            # TODO: finish this, but is it really required? might be nice to have
+
         # check if permissions assigned in cms are correct, and display
-        # a message if not - correctness mean: if user has add permisson to
-        # page, but he does'nt have auth permissions to add page object,
+        # a message if not - correctness mean: if user has add permission to
+        # page, but he doesn't have auth permissions to add page object,
         # display warning
         return self.cleaned_data
-    
+
     def save(self, commit=True):
         """
         Makes sure the boolean fields are set to False if they aren't
@@ -286,7 +422,7 @@ class PagePermissionInlineAdminForm(forms.ModelForm):
         if commit:
             instance.save()
         return instance
-    
+
     class Meta:
         model = PagePermission
 
@@ -300,7 +436,6 @@ class ViewRestrictionInlineAdminForm(PagePermissionInlineAdminForm):
 
 
 class GlobalPagePermissionAdminForm(forms.ModelForm):
-
     def clean(self):
         super(GlobalPagePermissionAdminForm, self).clean()
         if not self.cleaned_data['user'] and not self.cleaned_data['group']:
@@ -315,17 +450,17 @@ class GenericCmsPermissionForm(forms.ModelForm):
     can_change_page = forms.BooleanField(label=_('Change'), required=False, initial=True)
     can_delete_page = forms.BooleanField(label=_('Delete'), required=False)
     can_recover_page = forms.BooleanField(label=_('Recover (any) pages'), required=False)
-    
+
     # pageuser is for pageuser & group - they are combined together,
     # and read out from PageUser model
     can_add_pageuser = forms.BooleanField(label=_('Add'), required=False)
     can_change_pageuser = forms.BooleanField(label=_('Change'), required=False)
     can_delete_pageuser = forms.BooleanField(label=_('Delete'), required=False)
-    
+
     can_add_pagepermission = forms.BooleanField(label=_('Add'), required=False)
     can_change_pagepermission = forms.BooleanField(label=_('Change'), required=False)
     can_delete_pagepermission = forms.BooleanField(label=_('Delete'), required=False)
-    
+
     def populate_initials(self, obj):
         """Read out permissions from permission system.
         """
@@ -339,25 +474,27 @@ class GenericCmsPermissionForm(forms.ModelForm):
                 codename = getattr(model._meta, 'get_%s_permission' % t)()
                 initials['can_%s_%s' % (t, name)] = codename in permissions
         return initials
-    
+
+
 class PageUserForm(UserCreationForm, GenericCmsPermissionForm):
-    notify_user = forms.BooleanField(label=_('Notify user'), required=False, 
-        help_text=_('Send email notification to user about username or password change. Requires user email.'))
-    
+    notify_user = forms.BooleanField(label=_('Notify user'), required=False,
+                                     help_text=_(
+                                         'Send email notification to user about username or password change. Requires user email.'))
+
     class Meta:
         model = PageUser
-    
+
     def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                  initial=None, error_class=ErrorList, label_suffix=':',
                  empty_permitted=False, instance=None):
-        
+
         if instance:
             initial = initial or {}
             initial.update(self.populate_initials(instance))
-        
-        super(PageUserForm, self).__init__(data, files, auto_id, prefix, 
-            initial, error_class, label_suffix, empty_permitted, instance)
-        
+
+        super(PageUserForm, self).__init__(data, files, auto_id, prefix,
+                                           initial, error_class, label_suffix, empty_permitted, instance)
+
         if instance:
             # if it is a change form, keep those fields as not required
             # password will be changed only if there is something entered inside
@@ -365,20 +502,26 @@ class PageUserForm(UserCreationForm, GenericCmsPermissionForm):
             self.fields['password1'].label = _('New password')
             self.fields['password2'].required = False
             self.fields['password2'].label = _('New password confirmation')
-        
+
         self._password_change = True
-        
+
     def clean_username(self):
         if self.instance:
             return self.cleaned_data['username']
         return super(PageUserForm, self).clean_username()
-    
-    def clean_password2(self): 
+
+    # required if the User model's USERNAME_FIELD is the email field
+    def clean_email(self):
+        if self.instance:
+            return self.cleaned_data['email']
+        return super(PageUserForm, self).clean_email()
+
+    def clean_password2(self):
         if self.instance and self.cleaned_data['password1'] == '' and self.cleaned_data['password2'] == '':
             self._password_change = False
             return u''
         return super(PageUserForm, self).clean_password2()
-    
+
     def clean(self):
         cleaned_data = super(PageUserForm, self).clean()
         notify_user = self.cleaned_data['notify_user']
@@ -391,14 +534,14 @@ class PageUserForm(UserCreationForm, GenericCmsPermissionForm):
         if self.cleaned_data['can_add_pagepermission'] and not self.cleaned_data['can_change_pagepermission']:
             raise forms.ValidationError(_("To add permissions you also need to edit them!"))
         return cleaned_data
-    
+
     def save(self, commit=True):
         """Create user, assign him to staff users, and create permissions for 
         him if required. Also assigns creator to user.
         """
-        Super = self._password_change and PageUserForm or UserCreationForm  
+        Super = self._password_change and PageUserForm or UserCreationForm
         user = super(Super, self).save(commit=False)
-        
+
         user.is_staff = True
         created = not bool(user.pk)
         # assign creator to user
@@ -411,28 +554,27 @@ class PageUserForm(UserCreationForm, GenericCmsPermissionForm):
         if self.cleaned_data['notify_user']:
             mail_page_user_change(user, created, self.cleaned_data['password1'])
         return user
-    
-    
+
+
 class PageUserGroupForm(GenericCmsPermissionForm):
-    
     class Meta:
         model = PageUserGroup
         fields = ('name', )
-        
+
     def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                  initial=None, error_class=ErrorList, label_suffix=':',
                  empty_permitted=False, instance=None):
-        
+
         if instance:
             initial = initial or {}
             initial.update(self.populate_initials(instance))
-        
-        super(PageUserGroupForm, self).__init__(data, files, auto_id, prefix, 
-            initial, error_class, label_suffix, empty_permitted, instance)
-    
+
+        super(PageUserGroupForm, self).__init__(data, files, auto_id, prefix,
+                                                initial, error_class, label_suffix, empty_permitted, instance)
+
     def save(self, commit=True):
         group = super(GenericCmsPermissionForm, self).save(commit=False)
-        
+
         created = not bool(group.pk)
         # assign creator to user
         if created:
