@@ -7,30 +7,33 @@ You must implement the necessary permission checks in your own code before
 calling these methods!
 """
 import datetime
-from cms.utils.conf import get_cms_setting
-from django.core.exceptions import PermissionDenied
-from cms.utils.i18n import get_language_list
+from cms.constants import TEMPLATE_INHERITANCE_MAGIC
 
-from django.conf import settings
-from django.contrib.auth.models import User
 from django.contrib.sites.models import Site
-from django.db.models import Max
+from django.core.exceptions import FieldError
+from django.core.exceptions import PermissionDenied
+from django.core.exceptions import ValidationError
 from django.template.defaultfilters import slugify
-from menus.menu_pool import menu_pool
 
 from cms.admin.forms import save_permissions
 from cms.app_base import CMSApp
 from cms.apphook_pool import apphook_pool
+from cms.compat import get_user_model
 from cms.models.pagemodel import Page
-from cms.models.permissionmodels import (PageUser, PagePermission,
-    GlobalPagePermission, ACCESS_PAGE_AND_DESCENDANTS)
+from cms.models.permissionmodels import PageUser, PagePermission, \
+    GlobalPagePermission, ACCESS_PAGE_AND_DESCENDANTS
 from cms.models.placeholdermodel import Placeholder
 from cms.models.pluginmodel import CMSPlugin
 from cms.models.titlemodels import Title
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
-from cms.utils import moderator
+from cms.utils import copy_plugins
+from cms.utils.compat.type_checks import string_types
+from cms.utils.conf import get_cms_setting
+from cms.utils.i18n import get_language_list
 from cms.utils.permissions import _thread_locals
+from django.template.loader import get_template
+from menus.menu_pool import menu_pool
 
 
 #===============================================================================
@@ -45,6 +48,7 @@ VISIBILITY_STAFF = 2
 # Helpers/Internals
 #===============================================================================
 
+
 def _generate_valid_slug(source, parent, language):
     """
     Generate a valid slug for a page from source for the given language.
@@ -64,20 +68,32 @@ def _generate_valid_slug(source, parent, language):
         i += 1
     return slug
 
-def _verify_apphook(apphook):
+
+def _verify_apphook(apphook, namespace):
     """
     Verifies the apphook given is valid and returns the normalized form (name)
     """
-    if hasattr(apphook, '__module__') and  issubclass(apphook, CMSApp):
-        apphook_pool.discover_apps()
-        assert apphook in apphook_pool.apps.values()
+    apphook_pool.discover_apps()
+    if hasattr(apphook, '__module__') and issubclass(apphook, CMSApp):
+        try:
+            assert apphook in apphook_pool.apps.values()
+        except AssertionError:
+            print(apphook_pool.apps.values())
+            raise
         return apphook.__name__
-    elif isinstance(apphook, basestring):
-        apphook_pool.discover_apps()
-        assert apphook in apphook_pool.apps
-        return apphook
+    elif isinstance(apphook, string_types):
+        try:
+            assert apphook in apphook_pool.apps
+        except AssertionError:
+            print(apphook_pool.apps.values())
+            raise
+        apphook_name = apphook
     else:
         raise TypeError("apphook must be string or CMSApp instance")
+    if apphook_pool.apps[apphook_name].app_name and not namespace:
+        raise ValidationError('apphook with app_name must define a namespace')
+    return apphook_name
+
 
 def _verify_plugin_type(plugin_type):
     """
@@ -86,10 +102,11 @@ def _verify_plugin_type(plugin_type):
     """
     if (hasattr(plugin_type, '__module__') and
         issubclass(plugin_type, CMSPluginBase)):
+        plugin_pool.set_plugin_meta()
         plugin_model = plugin_type.model
         assert plugin_type in plugin_pool.plugins.values()
         plugin_type = plugin_type.__name__
-    elif isinstance(plugin_type, basestring):
+    elif isinstance(plugin_type, string_types):
         try:
             plugin_model = plugin_pool.get_plugin(plugin_type).model
         except KeyError:
@@ -105,27 +122,30 @@ def _verify_plugin_type(plugin_type):
 #===============================================================================
 
 def create_page(title, template, language, menu_title=None, slug=None,
-                apphook=None, redirect=None, meta_description=None,
-                meta_keywords=None, created_by='python-api', parent=None,
+                apphook=None, apphook_namespace=None, redirect=None, meta_description=None,
+                created_by='python-api', parent=None,
                 publication_date=None, publication_end_date=None,
                 in_navigation=False, soft_root=False, reverse_id=None,
                 navigation_extenders=None, published=False, site=None,
                 login_required=False, limit_visibility_in_menu=VISIBILITY_ALL,
-                position="last-child", overwrite_url=None):
+                position="last-child", overwrite_url=None, xframe_options=Page.X_FRAME_OPTIONS_INHERIT):
     """
     Create a CMS Page and it's title for the given language
     
     See docs/extending_cms/api_reference.rst for more info
     """
     # ugly permissions hack
-    if created_by and isinstance(created_by, User):
+    if created_by and isinstance(created_by, get_user_model()):
         _thread_locals.user = created_by
-        created_by = created_by.username
+
+        created_by = getattr(created_by, get_user_model().USERNAME_FIELD)
     else:
         _thread_locals.user = None
-    
+
     # validate template
-    assert template in [tpl[0] for tpl in get_cms_setting('TEMPLATES')]
+    if not template == TEMPLATE_INHERITANCE_MAGIC:
+        assert template in [tpl[0] for tpl in get_cms_setting('TEMPLATES')]
+        get_template(template)
 
     # validate site
     if not site:
@@ -135,17 +155,11 @@ def create_page(title, template, language, menu_title=None, slug=None,
 
     # validate language:
     assert language in get_language_list(site), get_cms_setting('LANGUAGES').get(site.pk)
-    
+
     # set default slug:
     if not slug:
         slug = _generate_valid_slug(title, parent, language)
-    
-    # validate and normalize apphook 
-    if apphook:
-        application_urls = _verify_apphook(apphook)
-    else:
-        application_urls = None
-    
+
     # validate parent
     if parent:
         assert isinstance(parent, Page)
@@ -154,26 +168,32 @@ def create_page(title, template, language, menu_title=None, slug=None,
     # validate publication date
     if publication_date:
         assert isinstance(publication_date, datetime.date)
-    
+
     # validate publication end date
     if publication_end_date:
         assert isinstance(publication_end_date, datetime.date)
-        
-    # validate softroot
-    assert get_cms_setting('SOFTROOT') or not soft_root
-    
+
     if navigation_extenders:
         raw_menus = menu_pool.get_menus_by_attribute("cms_enabled", True)
         menus = [menu[0] for menu in raw_menus]
         assert navigation_extenders in menus
-        
+
     # validate menu visibility
     accepted_limitations = (VISIBILITY_ALL, VISIBILITY_USERS, VISIBILITY_STAFF)
     assert limit_visibility_in_menu in accepted_limitations
-    
+
     # validate position
     assert position in ('last-child', 'first-child', 'left', 'right')
-    
+    # validate and normalize apphook
+    if apphook:
+        application_urls = _verify_apphook(apphook, apphook_namespace)
+    else:
+        application_urls = None
+
+    if reverse_id:
+        if Page.objects.drafts().filter(reverse_id=reverse_id).count():
+            raise FieldError('A page with the reverse_id="%s" already exist.' % reverse_id)
+
     page = Page(
         created_by=created_by,
         changed_by=created_by,
@@ -184,11 +204,13 @@ def create_page(title, template, language, menu_title=None, slug=None,
         soft_root=soft_root,
         reverse_id=reverse_id,
         navigation_extenders=navigation_extenders,
-        published=False, # will be published later
         template=template,
+        application_urls=application_urls,
+        application_namespace=apphook_namespace,
         site=site,
         login_required=login_required,
         limit_visibility_in_menu=limit_visibility_in_menu,
+        xframe_options=xframe_options,    
     )
     page.insert_at(parent, position)
     page.save()
@@ -198,23 +220,22 @@ def create_page(title, template, language, menu_title=None, slug=None,
         title=title,
         menu_title=menu_title,
         slug=slug,
-        apphook=application_urls,
         redirect=redirect,
         meta_description=meta_description,
-        meta_keywords=meta_keywords,
         page=page,
-        overwrite_url=overwrite_url
+        overwrite_url=overwrite_url,
     )
 
     if published:
-        page.publish()
+        page.publish(language)
 
     del _thread_locals.user
     return page.reload()
-    
+
+
 def create_title(language, title, page, menu_title=None, slug=None,
-                 apphook=None, redirect=None, meta_description=None,
-                 meta_keywords=None, parent=None, overwrite_url=None):
+                 redirect=None, meta_description=None,
+                 parent=None, overwrite_url=None):
     """
     Create a title.
     
@@ -231,22 +252,14 @@ def create_title(language, title, page, menu_title=None, slug=None,
     # set default slug:
     if not slug:
         slug = _generate_valid_slug(title, parent, language)
-        
-    # validate and normalize apphook 
-    if apphook:
-        application_urls = _verify_apphook(apphook)
-    else:
-        application_urls = None
-    
+
     title = Title.objects.create(
         language=language,
         title=title,
         menu_title=menu_title,
         slug=slug,
-        application_urls=application_urls,
         redirect=redirect,
         meta_description=meta_description,
-        meta_keywords=meta_keywords,
         page=page
     )
 
@@ -257,6 +270,7 @@ def create_title(language, title, page, menu_title=None, slug=None,
 
     return title
 
+
 def add_plugin(placeholder, plugin_type, language, position='last-child',
                target=None, **data):
     """
@@ -266,30 +280,43 @@ def add_plugin(placeholder, plugin_type, language, position='last-child',
     """
     # validate placeholder
     assert isinstance(placeholder, Placeholder)
-    
+
     # validate and normalize plugin type
     plugin_model, plugin_type = _verify_plugin_type(plugin_type)
-        
-
-    max_pos = CMSPlugin.objects.filter(language=language,
-        placeholder=placeholder).aggregate(Max('position'))['position__max'] or 0
+    if target:
+        if position == 'last-child':
+            new_pos = CMSPlugin.objects.filter(language=language, parent=target, tree_id=target.tree_id).count()
+        elif position == 'first-child':
+            new_pos = 0
+        elif position == 'left':
+            new_pos = target.position
+        elif position == 'right':
+            new_pos = target.position + 1
+        else:
+            raise Exception('position not supported: %s' % position)
+        for pl in CMSPlugin.objects.filter(language=language, parent=target.parent_id, tree_id=target.tree_id, position__gte=new_pos):
+            pl.position += 1
+            pl.save()
+    else:
+        new_pos = CMSPlugin.objects.filter(language=language, parent__isnull=True, placeholder=placeholder).count()
 
     plugin_base = CMSPlugin(
         plugin_type=plugin_type,
-        placeholder=placeholder, 
-        position=max_pos + 1,
+        placeholder=placeholder,
+        position=new_pos,
         language=language
     )
     plugin_base.insert_at(target, position=position, save=False)
-            
+
     plugin = plugin_model(**data)
     plugin_base.set_base_attr(plugin)
     plugin.save()
     return plugin
-    
+
+
 def create_page_user(created_by, user,
                      can_add_page=True, can_view_page=True,
-                     can_change_page=True, can_delete_page=True, 
+                     can_change_page=True, can_delete_page=True,
                      can_recover_page=True, can_add_pageuser=True,
                      can_change_pageuser=True, can_delete_pageuser=True,
                      can_add_pagepermission=True,
@@ -304,19 +331,19 @@ def create_page_user(created_by, user,
         # just be lazy
         return create_page_user(created_by, user, True, True, True, True,
                                 True, True, True, True, True, True, True)
-    
+
     # validate created_by
-    assert isinstance(created_by, User)
-    
+    assert isinstance(created_by, get_user_model())
+
     data = {
-        'can_add_page': can_add_page, 
-        'can_view_page': can_view_page, 
-        'can_change_page': can_change_page, 
-        'can_delete_page': can_delete_page, 
-        'can_recover_page': can_recover_page, 
-        'can_add_pageuser': can_add_pageuser, 
-        'can_change_pageuser': can_change_pageuser, 
-        'can_delete_pageuser': can_delete_pageuser, 
+        'can_add_page': can_add_page,
+        'can_view_page': can_view_page,
+        'can_change_page': can_change_page,
+        'can_delete_page': can_delete_page,
+        'can_recover_page': can_recover_page,
+        'can_add_pageuser': can_add_pageuser,
+        'can_change_pageuser': can_change_pageuser,
+        'can_delete_pageuser': can_delete_pageuser,
         'can_add_pagepermission': can_add_pagepermission,
         'can_change_pagepermission': can_change_pagepermission,
         'can_delete_pagepermission': can_delete_pagepermission,
@@ -324,19 +351,20 @@ def create_page_user(created_by, user,
     user.is_staff = True
     user.is_active = True
     page_user = PageUser(created_by=created_by)
-    for field in [f.name for f in User._meta.local_fields]:
+    for field in [f.name for f in get_user_model()._meta.local_fields]:
         setattr(page_user, field, getattr(user, field))
     user.save()
     page_user.save()
     save_permissions(data, page_user)
     return user
-        
+
+
 def assign_user_to_page(page, user, grant_on=ACCESS_PAGE_AND_DESCENDANTS,
-    can_add=False, can_change=False, can_delete=False, 
-    can_change_advanced_settings=False, can_publish=False, 
-    can_change_permissions=False, can_move_page=False,
-    can_recover_page=True, can_view=False,
-    grant_all=False, global_permission=False):
+                        can_add=False, can_change=False, can_delete=False,
+                        can_change_advanced_settings=False, can_publish=False,
+                        can_change_permissions=False, can_move_page=False,
+                        can_recover_page=True, can_view=False,
+                        grant_all=False, global_permission=False):
     """
     Assigns given user to page, and gives him requested permissions.
     
@@ -352,7 +380,7 @@ def assign_user_to_page(page, user, grant_on=ACCESS_PAGE_AND_DESCENDANTS,
         'can_change_permissions': can_change_permissions or grant_all,
         'can_move_page': can_move_page or grant_all,
         'can_view': can_view or grant_all,
-        }
+    }
 
     page_permission = PagePermission(page=page, user=user,
                                      grant_on=grant_on, **data)
@@ -363,8 +391,9 @@ def assign_user_to_page(page, user, grant_on=ACCESS_PAGE_AND_DESCENDANTS,
         page_permission.save()
         page_permission.sites.add(Site.objects.get_current())
     return page_permission
-    
-def publish_page(page, user):
+
+
+def publish_page(page, user, language):
     """
     Publish a page. This sets `page.published` to `True` and calls publish()
     which does the actual publishing.
@@ -372,13 +401,66 @@ def publish_page(page, user):
     See docs/extending_cms/api_reference.rst for more info
     """
     page = page.reload()
+
     class FakeRequest(object):
         def __init__(self, user):
             self.user = user
+
     request = FakeRequest(user)
     if not page.has_publish_permission(request):
         raise PermissionDenied()
-    page.published = True
-    page.save()
-    page.publish()
+    page.publish(language)
     return page.reload()
+
+
+def get_page_draft(page):
+    """
+    Returns the draft version of a page, regardless if the passed in
+    page is a published version or a draft version.
+
+    :param page: The page to get the draft version
+    :type page: :class:`cms.models.pagemodel.Page` instance
+    :return page: draft version of the page
+    :type page: :class:`cms.models.pagemodel.Page` instance
+    """
+    if page:
+        if page.publisher_is_draft:
+            return page
+        else:
+            return page.publisher_draft
+    else:
+        return None
+
+
+def copy_plugins_to_language(page, source_language, target_language,
+                             only_empty=True):
+    """
+    Copy the plugins to another language in the same page for all the page
+    placeholders.
+
+    By default plugins are copied only if placeholder has no plugin for the
+    target language; use ``only_empty=False`` to change this.
+
+    .. warning: This function skips permissions checks
+
+    :param page: the page to copy
+    :type page: :class:`cms.models.pagemodel.Page` instance
+    :param string source_language: The source language code,
+     must be in :setting:`django:LANGUAGES`
+    :param string target_language: The source language code,
+     must be in :setting:`django:LANGUAGES`
+    :param bool only_empty: if False, plugin are copied even if
+     plugins exists in the target language (on a placeholder basis).
+    :return int: number of copied plugins
+    """
+    copied = 0
+    placeholders = page.placeholders.all()
+    for placeholder in placeholders:
+        # only_empty is True we check if the placeholder already has plugins and
+        # we skip it if has some
+        if not only_empty or not placeholder.cmsplugin_set.filter(language=target_language).exists():
+            plugins = list(
+                placeholder.cmsplugin_set.filter(language=source_language).order_by('tree_id', 'level', 'position'))
+            copied_plugins = copy_plugins.copy_plugins_to(plugins, placeholder, target_language)
+            copied += len(copied_plugins)
+    return copied
