@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 from django.contrib.sitemaps import Sitemap
+from django.utils import translation
+from cms.models import Title
+
 
 def from_iterable(iterables):
     """
@@ -9,21 +12,25 @@ def from_iterable(iterables):
         for element in it:
             yield element
 
+
 class CMSSitemap(Sitemap):
     changefreq = "monthly"
     priority = 0.5
 
     def items(self):
-        from cms.utils.page_resolver import get_page_queryset
-        page_queryset = get_page_queryset(None)
-        all_pages = page_queryset.published().filter(login_required=False)
-        return all_pages
+        all_titles = Title.objects.public().filter(page__login_required=False)
+        return all_titles
 
-    def lastmod(self, page):
-        modification_dates = [page.changed_date, page.publication_date]
+    def lastmod(self, title):
+        modification_dates = [title.page.changed_date, title.page.publication_date]
         plugins_for_placeholder = lambda placeholder: placeholder.get_plugins()
-        plugins = from_iterable(map(plugins_for_placeholder, page.placeholders.all()))
+        plugins = from_iterable(map(plugins_for_placeholder, title.page.placeholders.all()))
         plugin_modification_dates = map(lambda plugin: plugin.changed_date, plugins)
         modification_dates.extend(plugin_modification_dates)
         return max(modification_dates)
-    
+
+    def location(self, title):
+        translation.activate(title.language)
+        url = title.page.get_absolute_url(title.language)
+        translation.deactivate()
+        return url
