@@ -1,91 +1,131 @@
 # -*- coding: utf-8 -*-
+from distutils.version import LooseVersion
+
 from classytags.arguments import Argument
 from classytags.core import Options, Tag
 from classytags.helpers import InclusionTag
-from cms.models import MASK_PAGE, MASK_CHILDREN, MASK_DESCENDANTS
+from cms.constants import PUBLISHER_STATE_PENDING
 from cms.utils import get_cms_setting
 from cms.utils.admin import get_admin_menu_item_context
-from cms.utils.i18n import get_language_object
 from cms.utils.permissions import get_any_page_view_permissions
-from distutils.version import LooseVersion
 from django import template
 from django.conf import settings
+from cms.utils.compat.dj import force_unicode
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext, ugettext_lazy as _
+from django.utils.translation import ugettext
 import django
+from django.utils.translation import ugettext_lazy as _
+
 
 register = template.Library()
 
-
 if LooseVersion(django.get_version()) < LooseVersion('1.4'):
     CMS_ADMIN_ICON_BASE = "%sadmin/img/admin/" % settings.STATIC_URL
 else:
     CMS_ADMIN_ICON_BASE = "%sadmin/img/" % settings.STATIC_URL
 
+
 class ShowAdminMenu(InclusionTag):
     name = 'show_admin_menu'
-    template = 'admin/cms/page/menu.html'
-    
+    template = 'admin/cms/page/tree/menu.html'
+
     options = Options(
         Argument('page')
     )
-    
+
     def get_context(self, context, page):
         request = context['request']
 
-        
         if context.has_key("cl"):
             filtered = context['cl'].is_filtered()
         elif context.has_key('filtered'):
             filtered = context['filtered']
-        
-        
-        
+        language = context['preview_language']
+
+
         # following function is newly used for getting the context per item (line)
         # if something more will be required, then get_admin_menu_item_context
         # function have to be updated. 
         # This is done because item can be reloaded after some action over ajax.
-        context.update(get_admin_menu_item_context(request, page, filtered))
-        
-        # this here is just context specific for menu rendering - items itself does
-        # not use any of following variables
-        #context.update({
-        #    'no_children': no_children,
-        #})
+        context.update(get_admin_menu_item_context(request, page, filtered, language))
         return context
+
+
 register.tag(ShowAdminMenu)
 
+
+class TreePublishRow(Tag):
+    name = "tree_publish_row"
+    options = Options(
+        Argument('page'),
+        Argument('language')
+    )
+
+    def render_tag(self, context, page, language):
+        if page.is_published(language) and page.publisher_public_id and page.publisher_public.is_published(language):
+            if page.is_dirty(language):
+                cls = "dirty"
+                text = _("unpublished changes")
+            else:
+                cls = "published"
+                text = _("published")
+        else:
+
+            if language in page.languages:
+                public_pending = page.publisher_public_id and page.publisher_public.get_publisher_state(
+                        language) == PUBLISHER_STATE_PENDING
+                if public_pending or page.get_publisher_state(
+                        language) == PUBLISHER_STATE_PENDING:
+                    cls = "unpublishedparent"
+                    text = _("unpublished parent")
+                else:
+                    cls = "unpublished"
+                    text = _("unpublished")
+            else:
+                cls = "empty"
+                text = _("no content")
+        return mark_safe('<span class="%s" title="%s"></span>' % (cls, force_unicode(text)))
+
+
+register.tag(TreePublishRow)
+
+
+@register.filter
+def is_published(page, language):
+    if page.is_published(language) and page.publisher_public_id and page.publisher_public.is_published(language):
+        return True
+    else:
+        if language in page.languages and page.publisher_public_id and page.publisher_public.get_publisher_state(
+                language) == PUBLISHER_STATE_PENDING:
+            return True
+        return False
+
+
 class ShowLazyAdminMenu(InclusionTag):
     name = 'show_lazy_admin_menu'
-    template = 'admin/cms/page/lazy_child_menu.html'
-    
+    template = 'admin/cms/page/tree/lazy_child_menu.html'
+
     options = Options(
         Argument('page')
     )
-    
+
     def get_context(self, context, page):
         request = context['request']
 
-        
         if context.has_key("cl"):
             filtered = context['cl'].is_filtered()
         elif context.has_key('filtered'):
             filtered = context['filtered']
-        
-        
-        
+
+        language = context['preview_language']
         # following function is newly used for getting the context per item (line)
         # if something more will be required, then get_admin_menu_item_context
         # function have to be updated. 
         # This is done because item can be reloaded after some action over ajax.
-        context.update(get_admin_menu_item_context(request, page, filtered))
-        
-        # this here is just context specific for menu rendering - items itself does
-        # not use any of following variables
-        #context.update({
-        #    'no_children': no_children,
-        #})
+        context.update(get_admin_menu_item_context(request, page, filtered, language))
         return context
+
+
 register.tag(ShowLazyAdminMenu)
 
 
@@ -96,12 +136,12 @@ class CleanAdminListFilter(InclusionTag):
     """
     name = 'clean_admin_list_filter'
     template = 'admin/filter.html'
-    
+
     options = Options(
         Argument('cl'),
         Argument('spec'),
     )
-    
+
     def get_context(self, context, cl, spec):
         choices = sorted(list(spec.choices(cl)), key=lambda k: k['query_string'])
         query_string = None
@@ -110,31 +150,35 @@ class CleanAdminListFilter(InclusionTag):
             if choice['query_string'] != query_string:
                 unique_choices.append(choice)
                 query_string = choice['query_string']
-        return {'title': spec.title, 'choices' : unique_choices}
-register.tag(CleanAdminListFilter)
+        return {'title': spec.title, 'choices': unique_choices}
 
 
+register.tag(CleanAdminListFilter)
+
 
 @register.filter
 def boolean_icon(value):
     BOOLEAN_MAPPING = {True: 'yes', False: 'no', None: 'unknown'}
-    return mark_safe(u'<img src="%sicon-%s.gif" alt="%s" />' % (CMS_ADMIN_ICON_BASE, BOOLEAN_MAPPING.get(value, 'unknown'), value))
+    return mark_safe(
+        u'<img src="%sicon-%s.gif" alt="%s" />' % (CMS_ADMIN_ICON_BASE, BOOLEAN_MAPPING.get(value, 'unknown'), value))
+
 
 @register.filter
 def is_restricted(page, request):
     if get_cms_setting('PERMISSION'):
-        all_perms = list(get_any_page_view_permissions(request, page))
-        icon = boolean_icon(bool(all_perms))
+        if hasattr(page, 'permission_restricted'):
+            icon = boolean_icon(bool(page.permission_restricted))
+        else:
+            all_perms = list(get_any_page_view_permissions(request, page))
+            icon = boolean_icon(bool(all_perms))
         return mark_safe(
-            ugettext('<span title="Restrictions: %(title)s">%(icon)s</span>') % {
-                'title': u', '.join((perm.get_grant_on_display() for perm in all_perms)) or None,
+            ugettext('<span>%(icon)s</span>') % {
                 'icon': icon,
             })
     else:
         icon = boolean_icon(None)
         return mark_safe(
-            ugettext('<span title="Restrictions: %(title)s">%(icon)s</span>') % {
-                'title': None,
+            ugettext('<span>%(icon)s</span>') % {
                 'icon': icon,
             })
 
@@ -142,7 +186,6 @@ def is_restricted(page, request):
 @register.filter
 def preview_link(page, language):
     if settings.USE_I18N:
-        from django.core.urlresolvers import reverse
 
         # Which one of page.get_slug() and page.get_path() is the right
         # one to use in this block? They both seem to return the same thing.
@@ -158,47 +201,54 @@ def preview_link(page, language):
 
 class RenderPlugin(InclusionTag):
     template = 'cms/content.html'
-    
+
     options = Options(
         Argument('plugin')
     )
-    
+
     def get_context(self, context, plugin):
         return {'content': plugin.render_plugin(context, admin=True)}
+
+
 register.tag(RenderPlugin)
 
 
 class PageSubmitRow(InclusionTag):
     name = 'page_submit_row'
-    template = 'admin/page_submit_line.html'
-    
+    template = 'admin/cms/page/submit_row.html'
+
     def get_context(self, context):
         opts = context['opts']
         change = context['change']
         is_popup = context['is_popup']
         save_as = context['save_as']
-        show_delete_translation = context.get('show_delete_translation')  
+        basic_info = context.get('advanced_settings', False)
+        advanced_settings = context.get('basic_info', False)
         language = context['language']
         return {
-            'onclick_attrib': (opts.get_ordered_objects() and change
-                                and 'onclick="submitOrderForm();"' or ''),
-            'show_delete_link': (not is_popup and context['has_delete_permission']
-                                  and (change or context['show_delete'])),
+            # TODO check this (old code: opts.get_ordered_objects() )
+            'onclick_attrib': (opts and change
+                               and 'onclick="submitOrderForm();"' or ''),
+            'show_delete_link': False,
             'show_save_as_new': not is_popup and change and save_as,
-            'show_save_and_add_another': context['has_add_permission'] and 
-                                not is_popup and (not save_as or context['add']),
+            'show_save_and_add_another': False,
             'show_save_and_continue': not is_popup and context['has_change_permission'],
             'is_popup': is_popup,
+            'basic_info': basic_info,
+            'advanced_settings': advanced_settings,
             'show_save': True,
             'language': language,
-            'language_name': get_language_object(language)['name'],
-            'show_delete_translation': show_delete_translation
+            'object_id': context.get('object_id', None)
         }
+
+
 register.tag(PageSubmitRow)
 
 
 def in_filtered(seq1, seq2):
     return [x for x in seq1 if x in seq2]
+
+
 in_filtered = register.filter('in_filtered', in_filtered)
 
 
@@ -216,4 +266,5 @@ class CMSAdminIconBase(Tag):
     def render_tag(self, context):
         return CMS_ADMIN_ICON_BASE
 
+
 register.tag(CMSAdminIconBase)
