@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
 from cms.api import create_page
-from cms.models.pagemodel import Page
 from cms.test_utils.util.context_managers import SettingsOverride
 
 
@@ -28,10 +27,10 @@ class MenusFixture(object):
             p4 = create_page('P4', published=True, in_navigation=True, **defaults)
             p6 = create_page('P6', published=True, in_navigation=False, **defaults)
             p2 = create_page('P2', published=True, in_navigation=True, parent=p1, **defaults)
-            p3 = create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
-            p5 = create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
-            p7 = create_page('P7', published=True, in_navigation=True, parent=p6, **defaults)
-            p8 = create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
+            create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
+            create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
+            create_page('P7', published=True, in_navigation=True, parent=p6, **defaults)
+            create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
 
 
 class ExtendedMenusFixture(object):
@@ -60,13 +59,13 @@ class ExtendedMenusFixture(object):
             p4 = create_page('P4', published=True, in_navigation=True, **defaults)
             p6 = create_page('P6', published=True, in_navigation=False, **defaults)
             p2 = create_page('P2', published=True, in_navigation=True, parent=p1, **defaults)
-            p3 = create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
-            p5 = create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
-            p7 = create_page('P7', published=True, in_navigation=True, parent=p6, **defaults)
-            p6 = create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
+            create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
+            create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
+            create_page('P7', published=True, in_navigation=True, parent=p6, **defaults)
+            create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
             p9 = create_page('P9', published=True, in_navigation=True, parent=p1, **defaults)
             p10 = create_page('P10', published=True, in_navigation=True, parent=p9, **defaults)
-            p11 = create_page('P11', published=True, in_navigation=True, parent=p10, **defaults)
+            create_page('P11', published=True, in_navigation=True, parent=p10, **defaults)
 
 
 class SubMenusFixture(object):
@@ -92,10 +91,10 @@ class SubMenusFixture(object):
             p4 = create_page('P4', published=True, in_navigation=True, **defaults)
             p6 = create_page('P6', published=True, in_navigation=True, **defaults)
             p2 = create_page('P2', published=True, in_navigation=True, parent=p1, **defaults)
-            p3 = create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
-            p5 = create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
-            p7 = create_page('P7', published=True, in_navigation=False, parent=p6, **defaults)
-            p8 = create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
+            create_page('P3', published=True, in_navigation=True, parent=p2, **defaults)
+            create_page('P5', published=True, in_navigation=True, parent=p4, **defaults)
+            create_page('P7', published=True, in_navigation=False, parent=p6, **defaults)
+            create_page('P8', published=True, in_navigation=True, parent=p6, **defaults)
 
 
 class SoftrootFixture(object):
@@ -126,7 +125,7 @@ class SoftrootFixture(object):
             aaa = create_page('aaa', parent=root, **defaults)
             _111 = create_page('111', parent=aaa, **defaults)
             ccc = create_page('ccc', parent=_111, **defaults)
-            ddd = create_page('ddd', parent=ccc, **defaults)
+            create_page('ddd', parent=ccc, **defaults)
             create_page('222', parent=aaa, **defaults)
             bbb = create_page('bbb', parent=root, **defaults)
             create_page('333', parent=bbb, **defaults)
