@@ -1,7 +1,6 @@
 from cms.utils.conf import get_cms_setting
 from django.conf import settings
-from django.conf.urls.defaults import handler500, handler404, patterns, include, \
-    url
+from django.conf.urls import patterns, include, url
 
 
 urlpatterns = patterns('',
@@ -12,3 +11,10 @@ urlpatterns = patterns('',
         {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),
     url(r'^', include('cms.urls')),
 )
+
+
+if settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS:
+    import debug_toolbar
+    urlpatterns += patterns('',
+        url(r'^__debug__/', include(debug_toolbar.urls)),
+    )
