@@ -1,7 +1,6 @@
 from cms.utils import get_cms_setting
 from django.conf import settings
-from django.conf.urls.defaults import handler500, handler404, patterns, include, \
-    url
+from django.conf.urls import patterns, include, url
 try:
     from django.conf.urls.i18n import i18n_patterns
 except ImportError:
@@ -22,3 +21,10 @@ urlpatterns += i18n_patterns('',
     url(r'^admin/', include(admin.site.urls)),
     url(r'^content/', include('cms.urls')),
 )
+
+
+if settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS:
+    import debug_toolbar
+    urlpatterns += patterns('',
+        url(r'^__debug__/', include(debug_toolbar.urls)),
+    )
