@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
+from cms.utils import get_language_from_request
 from cms.utils.i18n import force_language, hide_untranslated
 from django.conf import settings
+from django.core.urlresolvers import NoReverseMatch
 import warnings
 from cms.models.titlemodels import Title
 
@@ -79,7 +81,7 @@ class DefaultLanguageChanger(object):
     def app_path(self):
         if self._app_path is None:
             if settings.USE_I18N:
-                page_path = self.get_page_path(self.request.LANGUAGE_CODE)
+                page_path = self.get_page_path(get_language_from_request(self.request))
             else:
                 page_path = self.get_page_path(settings.LANGUAGE_CODE)
             if page_path:
@@ -89,6 +91,12 @@ class DefaultLanguageChanger(object):
         return self._app_path
 
     def __call__(self, lang):
+        if hasattr(self.request, 'toolbar') and self.request.toolbar.obj:
+            with force_language(lang):
+                try:
+                    return self.request.toolbar.obj.get_absolute_url()
+                except:
+                    pass
         return '%s%s' % (self.get_page_path(lang), self.app_path)
 
     def get_page_path(self, lang):
@@ -97,7 +105,7 @@ class DefaultLanguageChanger(object):
             with force_language(lang):
                 try:
                     return page.get_absolute_url(language=lang, fallback=False)
-                except Title.DoesNotExist:
+                except (Title.DoesNotExist, NoReverseMatch):
                     if hide_untranslated(lang) and settings.USE_I18N:
                         return '/%s/' % lang
                     else:
